<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- XSLT stylesheet template fragment:
	Draws axes for a cartesian graph.
	The axes and their labels are drawn within a 100x100 area
	based at 0,0 in whatever the current viewbox may be.
	We reserve 5 units for the labels, and 10 for the axes and
	scale markings.
-->
<!-- $Id$ -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:template match="x-axis">
    <xsl:text>    </xsl:text>
    <g stroke="{@color}" fill="{@color}">
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates>
	<xsl:with-param name="rotate" select="0"/>
	<xsl:with-param name="base" select="15"/>
	<xsl:with-param name="dir" select="1"/>
	<xsl:with-param name="left" select="15"/>
	<xsl:with-param name="right" select="100"/>
	<xsl:with-param name="top" select="85"/>
	<xsl:with-param name="markalign" select="'middle'"/>
      </xsl:apply-templates>
      <xsl:text>    </xsl:text>
    </g>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="y-axis">
    <xsl:text>    </xsl:text>
    <g transform="translate(100,0) rotate(90)" stroke="{@color}"
	fill="{@color}">
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates>
	<xsl:with-param name="rotate" select="90"/>
	<xsl:with-param name="base" select="85"/>
	<xsl:with-param name="dir" select="-1"/>
	<xsl:with-param name="left" select="0"/>
	<xsl:with-param name="right" select="85"/>
	<xsl:with-param name="top" select="85"/>
	<xsl:with-param name="markalign" select="'end'"/>
      </xsl:apply-templates>
      <xsl:text>    </xsl:text>
    </g>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="label">
    <xsl:param name="left"/>
    <xsl:param name="right"/>
    <xsl:param name="rotate"/>
    <xsl:text>      </xsl:text>
    <text x="{$left + ($right - $left) div 2}" y="100" font-size="7"
	text-anchor="middle" stroke="none" fill="{@color}">
      <xsl:if test="string(@font) != ''">
	<xsl:attribute name="font-family">
	  <xsl:value-of select="@font"/>
	</xsl:attribute>
      </xsl:if>
      <!-- For the y-axis, flip the label upside down so it reads up. -->
      <xsl:if test="$rotate != 0">
	<xsl:attribute name="transform">rotate(180, <xsl:value-of
		select="$left + ($right - $left) div 2"/>,97.5)</xsl:attribute>
      </xsl:if>
      <xsl:value-of select="."/>
    </text>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="scale">
    <xsl:param name="rotate"/>
    <xsl:param name="base"/>
    <xsl:param name="dir"/>
    <xsl:param name="left"/>
    <xsl:param name="right"/>
    <xsl:param name="top"/>
    <xsl:param name="markalign"/>
    <xsl:param name="linewidth" select="1"/>
    <xsl:param name="markwidth" select="0.5"/>
    <xsl:param name="gridwidth" select="0.5"/>

    <xsl:text>      </xsl:text>
    <path d="M {$left},{$top} h {$right - $left}" stroke-width="{$linewidth}"/>
    <xsl:text>&#10;      </xsl:text>
    <g font-size="3" text-anchor="{$markalign}">
    <xsl:text>&#10;</xsl:text>
      <xsl:choose>
	<xsl:when test="@type = 'enum'">
	  <xsl:apply-templates>
	    <xsl:with-param name="unit" select="($right - $left) div @count"/>
	    <xsl:with-param name="base" select="$base"/>
	    <xsl:with-param name="dir" select="$dir"/>
	    <xsl:with-param name="top" select="$top"/>
	    <xsl:with-param name="markwidth" select="$markwidth"/>
	    <xsl:with-param name="gridwidth" select="$gridwidth"/>
	  </xsl:apply-templates>
	</xsl:when>
	<xsl:when test="@type = 'numeric'">
	  <xsl:call-template name="numeric-scale">
	    <xsl:with-param name="rotate" select="$rotate"/>
	    <xsl:with-param name="unit" select="($right - $left) div @range"/>
	    <xsl:with-param name="base" select="$base"/>
	    <xsl:with-param name="dir" select="$dir"/>
	    <xsl:with-param name="top" select="$top"/>
	    <xsl:with-param name="markalign" select="$markalign"/>
	    <xsl:with-param name="mark" select="@low"/>
	    <xsl:with-param name="markwidth" select="$markwidth"/>
	    <xsl:with-param name="gridwidth" select="$gridwidth"/>
	  </xsl:call-template>
	</xsl:when>
      </xsl:choose>
      <xsl:text>      </xsl:text>
    </g>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="enum">
    <xsl:param name="unit"/>
    <xsl:param name="base"/>
    <xsl:param name="dir"/>
    <xsl:param name="top"/>
    <xsl:param name="markwidth"/>
    <xsl:param name="gridwidth"/>
    <xsl:variable name="x" select="$base + $dir*(position()-0.5)*$unit"/>
    <xsl:if test="position() != last()">
      <xsl:text>        </xsl:text>
      <path d="M {$x + ($unit div 2)},{$top} v 3" stroke-width="{$markwidth}"/>
      <xsl:text>&#10;</xsl:text>
      <xsl:if test="../@grid != '' and position() mod ../@grid = 0">
	<xsl:text>        </xsl:text>
	<path d="M {$x+($unit div 2)},0 v {$top}" stroke-width="{$gridwidth}"/>
	<xsl:text>&#10;</xsl:text>
      </xsl:if>
    </xsl:if>
    <xsl:text>        </xsl:text>
    <text x="{$x}" y="{$top + 7}" stroke="none">
      <xsl:value-of select="."/>
    </text>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template name="numeric-scale">
    <xsl:param name="rotate"/>
    <xsl:param name="base"/>
    <xsl:param name="unit"/>
    <xsl:param name="dir"/>
    <xsl:param name="top"/>
    <xsl:param name="markalign"/>
    <xsl:param name="mark"/>
    <xsl:param name="markwidth"/>
    <xsl:param name="gridwidth"/>

    <xsl:variable name="x" select="$base + $dir * (($mark - @low) * $unit)"/>
    <xsl:if test="$mark &lt;= @high">
      <xsl:text>        </xsl:text>
      <path d="M {$x},{$top} v 2" stroke-width="{$markwidth}"/>
      <xsl:text>&#10;</xsl:text>
      <xsl:if test="(round($mark div @quant) mod round(@numbers div @quant)) = 0">
	<xsl:text>        </xsl:text>
	<path d="M {$x},{$top} v 3" stroke-width="{$markwidth}"/>
	<xsl:text>&#10;        </xsl:text>
	<text x="{$x}" y="{$top + 6}" stroke="none">
	  <xsl:if test="$rotate != 0">
	    <xsl:attribute name="transform">rotate(-90, <xsl:value-of
		select="$x"/>, <xsl:value-of
		select="$top + 4.5"/>)</xsl:attribute>
	  </xsl:if>
	  <xsl:value-of select="$mark"/>
	  <xsl:if test="/graph/@style = 'percent'">
	    <xsl:text>%</xsl:text>
	  </xsl:if>
	</text>
	<xsl:text>&#10;</xsl:text>
      </xsl:if>
      <xsl:if test="@grid != '' and $mark mod @grid = 0">
	<xsl:text>        </xsl:text>
	<path d="M {$x},0 v {$top}" stroke-width="{$gridwidth}"/>
	<xsl:text>&#10;</xsl:text>
      </xsl:if>
      <xsl:call-template name="numeric-scale">
	<xsl:with-param name="rotate" select="$rotate"/>
	<xsl:with-param name="base" select="$base"/>
	<xsl:with-param name="unit" select="$unit"/>
	<xsl:with-param name="dir" select="$dir"/>
	<xsl:with-param name="top" select="$top"/>
	<xsl:with-param name="markalign" select="$markalign"/>
	<xsl:with-param name="mark" select="round(($mark+@ticks) div @quant) * @quant"/>
	<xsl:with-param name="markwidth" select="$markwidth"/>
	<xsl:with-param name="gridwidth" select="$gridwidth"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
