<?xml version="1.0" encoding="ISO-8859-1"?>
 
<!-- XSLT stylesheet template fragment:
	Converts top-level of GDML to SVG for any kind of graph.
	This is to be included within a conversion template to
	process the markup common to all graph types.
	Currently, this consists of the graph title, legend,
	and plot framework.
-->
<!-- $Id$ -->

<!-- Declare the stylesheet. -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

<!-- Declare the output method: XML. -->
  <xsl:output method="xml" indent="yes"
              doctype-public="-//W3C//DTD SVG 1.0//EN"
              doctype-system="http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd"
              version="1.0"
              media-type="image/svg+xml"
              encoding="UTF-8" />
  <xsl:strip-space elements="*"/>

<!-- Main template: matches the document element.
	This constructs the overall canvass, with a 100x100 viewbox
	(with one unit of border around the edges), and puts the title
	in the top 10 units.
 -->
  <xsl:template match="/">
    <svg width="{/graph/@width}" height="{/graph/@height}"
	 viewBox="-1 0 102 101" preserveAspectRatio="none">
      <xsl:if test="/graph/@type = 'pie'">
	<xsl:attribute name="preserveAspectRatio">xMidYMid</xsl:attribute>
      </xsl:if>
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates/>
    </svg>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="/graph">
    <xsl:text>  </xsl:text>
    <title><xsl:value-of select="title"/></title>
    <xsl:text>&#10;</xsl:text>
    <xsl:apply-templates select="background"/>
    <xsl:apply-templates select="title"/>
    <xsl:apply-templates select="legend"/>
    <xsl:apply-templates select="plot"/>
  </xsl:template>

<!-- Background processing.
	This will create the background.  It is the first template
	applied by the main template, so that everything else will
	be drawn on this.
-->
  <xsl:template match="background">
    <xsl:text>  </xsl:text>
    <xsl:choose>
      <xsl:when test="@color != ''">
	<path d="M -1,0 h 102 v 100 h -102 Z" fill="{@color}"/>
      </xsl:when>
      <xsl:when test="string(.) != ''">
	<path d="M -1,0 h 102 v 100 h -102 Z" fill="text()"/>
      </xsl:when>
    </xsl:choose>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

<!-- Title processing.
	If the title is too long, we define a new coordinate system that
	is wide enough to hold it.  This causes the title to be squished
	horizontally to fit the available space.
-->
  <xsl:template match="title">
    <xsl:text>  </xsl:text>
    <xsl:choose>
      <xsl:when test="string-length(.) > 20">
	<xsl:variable name="width" select="string-length(.) * 5"/>
	<svg x="0" y="0" height="10" width="102" viewBox="0 0 {$width} 10">
	  <text x="{$width div 2}" y="9" fill="{@color}"
		style="font-size:9; text-anchor:middle">
	    <xsl:value-of select="."/>
	  </text>
	</svg>
      </xsl:when>
      <xsl:otherwise>
	<text x="50" y="9" style="font-size:9; text-anchor:middle"
		fill="{@color}">
	  <xsl:if test="string(@font) != ''">
	    <xsl:attribute name="font-family">
	      <xsl:value-of select="@font"/>
	    </xsl:attribute>
	  </xsl:if>
	  <xsl:value-of select="."/>
	</text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

<!-- Define the area that will contain the plot.  Reserve space at the
	top for the title, and at the right for the legend.  Our target
	plotting area within the overall image is an 80x90 area starting
	at 0,10.  However, we want to allow some elements of the plot to
	extend outside of this, such as axes numbers that may be centered
	at the very end of the axis.  Instead of clipping such things, we
	want them to appear even if they are in the title or legend areas.
	So we define a slightly larger area, and arrange for the viewBox
	within it to have 0,0 to 100,100 overlayed at 0,10 to 80,100 of
	our current viewBox.
-->
  <xsl:template match="plot">
    <xsl:text>  </xsl:text>
    <svg x="0" y="5" width="85" height="95" viewBox="0 -6 107 107"
	 preserveAspectRatio="none">
      <xsl:if test="/graph/@type = 'pie'">
	<xsl:attribute name="preserveAspectRatio">xMidYMid</xsl:attribute>
      </xsl:if>
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates/>
      <xsl:text>  </xsl:text>
    </svg>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>
  
  <!-- Ignore all other elements and comments. -->
  <!-- Templates included after this one may cause some of these ignored
	elements to be used.
    -->
  <xsl:template match="legend"/>
  <xsl:template match="item"/>
  <xsl:template match="x-axis"/>
  <xsl:template match="y-axis"/>
  <xsl:template match="label"/>
  <xsl:template match="scale"/>
  <xsl:template match="data"/>
  <xsl:template match="row"/>
  <xsl:template match="col"/>
  <xsl:template match="comment()"/>

</xsl:stylesheet>
