<?xml version="1.0" encoding="ISO-8859-1"?>
 
<!-- XSLT stylesheet template to convert XML containing a result set and
	user input data into GDML.
 -->
<!-- $Id$ -->

<!-- Declare the stylesheet. -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

<!-- This parameter is set in the invoking XSLT processor to name the
	document containing the meta-data about the graph.
  -->
  <xsl:param name="metadatafile" select="'meta.xml'"/>

<!-- Define some global variables to name the other documents we read. -->
  <xsl:variable name="colormapdoc" select="document('colormap.xml')"/>
  <xsl:variable name="metadatadoc" select="document($metadatafile)"/>

<!-- Create a data lookup key for the colormap. -->
  <xsl:key name="colormapkey" match="color" use="@number"/>

<!-- Declare the output method: XML. -->
  <xsl:output method="xml" indent="yes"
              omit-xml-declaration="yes"
              version="1.0"
              encoding="UTF-8" />
  <xsl:strip-space elements="*"/>


<!-- Main template: matches the data document element. -->
  <xsl:template match="/">
    <!-- Start output with the GDML document element. -->
    <graph height="{$metadatadoc/metadata/size/@height}"
	width="{$metadatadoc/metadata/size/@width}"
	orientation="{$metadatadoc/metadata/orientation}"
	style="{$metadatadoc/metadata/style}"
	type="{$metadatadoc/metadata/type}">
      <xsl:text>&#10;</xsl:text>
      <!-- Emit graphic definitions, in the order they are to be rendered.
	   Start with the background, then put the title and legend on it.
	-->
      <xsl:apply-templates select="$metadatadoc/metadata/background"/>
      <xsl:apply-templates select="$metadatadoc/metadata/title"/>

      <!-- Output the legend, specifying what data it should represent.
	   The orientation of the legend is reversed for pie charts,
	   because they map colors to rows, not columns
        -->
      <xsl:choose>
	<xsl:when test="$metadatadoc/metadata/type = 'pie'">
	  <xsl:call-template name="legend">
	    <xsl:with-param name="data" select="RESULTS/ROW"/>
	  </xsl:call-template>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:call-template name="legend">
	    <xsl:with-param name="data" select="RESULTS/ROW[1]/COL"/>
	  </xsl:call-template>
	</xsl:otherwise>
      </xsl:choose>
      <!-- Output the plot area information. -->
      <xsl:text>  </xsl:text>
      <plot>
      <xsl:text>&#10;</xsl:text>
	<xsl:apply-templates select="$metadatadoc/metadata/x-axis" mode="axes">
	  <xsl:with-param name="data" select="RESULTS/ROW"/>
	</xsl:apply-templates>
	<xsl:apply-templates select="$metadatadoc/metadata/y-axis" mode="axes">
	  <xsl:with-param name="data" select="RESULTS/ROW/COL"/>
	</xsl:apply-templates>
	<xsl:apply-templates mode="data"/>
      <xsl:text>  </xsl:text>
      </plot>
      <xsl:text>&#10;</xsl:text>
    </graph>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- Output the title. -->
  <xsl:template match="title">
    <xsl:text>  </xsl:text>
    <title>
      <xsl:attribute name="color">
	<xsl:choose>
	  <xsl:when test="string(@color) != ''">
	    <xsl:value-of select="@color"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:text>black</xsl:text>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
      <xsl:if test="string(@font) != ''">
	<xsl:attribute name="font">
	  <xsl:value-of select="@font"/>
	</xsl:attribute>
      </xsl:if>
      <xsl:value-of select="."/>
    </title>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- Do the background. -->
  <xsl:template match="background">
    <xsl:text>  </xsl:text>
    <background color="{.}"/>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- This produces the legend. -->
  <xsl:template name="legend" mode="legend">
    <xsl:param name="data"/>
    <xsl:text>  </xsl:text>
    <legend>
      <xsl:text>&#10;</xsl:text>
      <xsl:for-each select="$data">
	<xsl:variable name="itemnum"
		select="(position() - 1) mod
			count($colormapdoc/colormap/color) + 1"/>
	<xsl:text>    </xsl:text>
	<item>
	  <xsl:choose>
	    <xsl:when test="$metadatadoc/metadata/type = 'pie'">
	      <xsl:attribute name="type">pie</xsl:attribute>
	    </xsl:when>
	    <xsl:when test="$metadatadoc/metadata/type = 'line'">
	      <xsl:attribute name="type">line</xsl:attribute>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:attribute name="type">block</xsl:attribute>
	    </xsl:otherwise>
	  </xsl:choose>
	  <xsl:attribute name="color">
	    <xsl:for-each select="$colormapdoc">
	      <xsl:value-of select="key('colormapkey', $itemnum)"/>
	    </xsl:for-each>
	  </xsl:attribute>
	  <xsl:choose>
	    <xsl:when test="@name">
	      <xsl:value-of select="@name"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:text>#</xsl:text>
	      <xsl:value-of select="position()"/>
	    </xsl:otherwise>
	  </xsl:choose>
	</item>
	<xsl:text>&#10;</xsl:text>
      </xsl:for-each>
    <xsl:text>  </xsl:text>
    </legend>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- The axes are done by calling the same template, but with
	different data items.
    -->
  <xsl:template match="x-axis" mode="axes">
    <xsl:param name="data"/>
      <xsl:call-template name="axis">
	<xsl:with-param name="data" select="$data"/>
      </xsl:call-template>
  </xsl:template>

  <xsl:template match="y-axis" mode="axes">
    <xsl:param name="data"/>
      <xsl:call-template name="axis">
	<xsl:with-param name="data" select="$data"/>
      </xsl:call-template>
  </xsl:template>

  <!-- This is the axis processing template. -->
  <xsl:template name="axis" mode="axes">
    <xsl:param name="data"/>
    <xsl:text>    </xsl:text>
    <xsl:element name="{name(.)}">
      <xsl:if test="string(@font) != ''">
	<xsl:attribute name="font">
	  <xsl:value-of select="@font"/>
	</xsl:attribute>
      </xsl:if>
      <xsl:attribute name="color">
	<xsl:value-of select="'black'"/>
      </xsl:attribute>
      <xsl:text>&#10;</xsl:text>
      <xsl:text>      </xsl:text>
      <label>
	<xsl:attribute name="color">
	  <xsl:if test="string(@color) = ''"><xsl:text>black</xsl:text></xsl:if>
	  <xsl:value-of select="@color"/>
	</xsl:attribute>
	<xsl:if test="string(@font) != ''">
	  <xsl:attribute name="font">
	    <xsl:value-of select="@font"/>
	  </xsl:attribute>
	</xsl:if>
	<xsl:value-of select="."/>
      </label>
      <xsl:text>&#10;</xsl:text>
      <xsl:choose>
	<xsl:when test="@type = 'enum'">
	  <xsl:variable name="count" select="count($data)"/>
	  <xsl:text>      </xsl:text>
	  <scale quant="1" grid="" numbers="{100 div $count}"
		ticks="{100 div $count}" high="100" low="0"
		range="100" count="{$count}" type="{@type}">
	    <xsl:text>&#10;</xsl:text>
	    <xsl:for-each select="$data">
	      <xsl:text>        </xsl:text>
	      <enum>
		<xsl:choose>
		  <xsl:when test="@name">
		    <xsl:value-of select="@name"/>
		  </xsl:when>
		  <xsl:otherwise>
		    <xsl:text>#</xsl:text>
		    <xsl:value-of select="position()"/>
		  </xsl:otherwise>
		</xsl:choose>
	      </enum>
	      <xsl:text>&#10;</xsl:text>
	    </xsl:for-each>
	    <xsl:text>      </xsl:text>
	  </scale>
	  <xsl:text>&#10;</xsl:text>
	</xsl:when>
	<xsl:when test="@type = 'numeric'">
	  <xsl:text>      </xsl:text>
	  <scale quant="{@quant}" grid="{@grid}" numbers="{@numbers}"
		 ticks="{@ticks}" high="{@high}" low="{@low}"
		 range="{@high - @low}" type="{@type}"/>
	  <xsl:text>&#10;</xsl:text>
	</xsl:when>
      </xsl:choose>
    <xsl:text>    </xsl:text>
    </xsl:element>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- The default action in data mode is to ignore everything that
	we do not explicitly match.  This lets us ignore the column
	definitions and other stuff we do not need that may appear in
	the input data.
    -->
  <xsl:template match="node()" mode="data"/>

  <!-- The data is passed through with some added attributes. -->
  <xsl:template match="/RESULTS" mode="data">
    <xsl:text>    </xsl:text>
    <data>
      <xsl:attribute name="rows">
	<xsl:value-of select="count(ROW)"/>
      </xsl:attribute>
      <xsl:attribute name="cols">
	<xsl:value-of select="count(ROW[1]/COL)"/>
      </xsl:attribute>
      <xsl:text>&#10;</xsl:text>
      <!-- Select the rows so that the first row is at position 1
	   regardless of what came before it.
	-->
      <xsl:apply-templates select="ROW" mode="data"/>
    <xsl:text>    </xsl:text>
    </data>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- Just pass each row definition along.  For pie charts, which use
	a different color for each row, we select a color based on the
	row position, and give that to the column template.
    -->
  <xsl:template match="/RESULTS/ROW" mode="data">
    <xsl:text>      </xsl:text>
    <xsl:variable name="colnum"
	select="(position() - 1) mod count($colormapdoc/colormap/color) + 1"/>
    <row name="{@name}">
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates mode="data">
	<xsl:with-param name="color">
	  <xsl:for-each select="$colormapdoc">
	    <xsl:value-of select="key('colormapkey',$colnum)"/>
	  </xsl:for-each>
	</xsl:with-param>
      </xsl:apply-templates>
      <xsl:text>      </xsl:text>
    </row>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- For each column, we copy its name, and assign a color from our
	colormap to it.  For pie charts, we use the color selected for
	the row.  We alter the data value if it is non-numeric or
	we are doing pie charts or percent charts and it is negative.
    -->
  <xsl:template match="/RESULTS/ROW/COL" mode="data">
    <xsl:param name="color"/>
    <xsl:text>        </xsl:text>
    <col name="{@name}">
      <xsl:attribute name="color">
	<xsl:choose>
	  <xsl:when test="$metadatadoc/metadata/type = 'pie'">
	    <xsl:value-of select="$color"/>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:variable name="colnum"
		select="(position() - 1) mod
				count($colormapdoc/colormap/color) + 1"/>
	    <xsl:for-each select="$colormapdoc">
	      <xsl:value-of select="key('colormapkey',$colnum)"/>
	    </xsl:for-each>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:attribute>
      <xsl:choose>
      <xsl:when test="string(number(text())) = 'NaN'">
	<!-- For any non-numeric values, mark them and replace with zero. -->
	<xsl:attribute name="nonnumeric"/>
	<xsl:text>0</xsl:text>
      </xsl:when>
	<!-- For all pie charts or any chart in a percent style,
	     strip any minus signs from the data in order to get
	     the absolute values of all data values.  We do it
	     here because we have to sum these values in the
	     next template, so we cannot change the sign there.
	  -->
	<xsl:when test="$metadatadoc/metadata/type = 'pie' or $metadatadoc/metadata/style = 'percent'">
	  <xsl:value-of select="translate(text(), '-', '')"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="text()"/>
	</xsl:otherwise>
      </xsl:choose>
    </col>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

</xsl:stylesheet>
