<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- XSLT stylesheet template fragment:
	Draws data columns for a column chart.
-->
<!-- $Id$ -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:template match="data">
    <!-- Shift data area right and up, above axes. -->
    <xsl:variable name="left" select="../x-axis/scale/@low"/>
    <xsl:variable name="top" select="../y-axis/scale/@low"/>
    <xsl:variable name="right" select="../x-axis/scale/@high"/>
    <xsl:variable name="bottom" select="../y-axis/scale/@high"/>
    <xsl:text>    </xsl:text>
    <svg x="15" y="0" width="85" height="85" stroke="black" stroke-width="0.01"
	viewBox="{$left} {$top} {$right - $left} {$bottom - $top}"
	preserveAspectRatio="none">
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates>
	<xsl:with-param name="left" select="$left"/>
	<xsl:with-param name="top" select="$top"/>
	<xsl:with-param name="right" select="$right"/>
	<xsl:with-param name="bottom" select="$bottom"/>
      </xsl:apply-templates>
      <xsl:text>    </xsl:text>
    </svg>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="row">
    <xsl:param name="left"/>
    <xsl:param name="top"/>
    <xsl:param name="right"/>
    <xsl:param name="bottom"/>
    <!-- Shift each row right (pos*width/nrows). -->
    <xsl:variable name="rowwidth" select="($right - $left) div ../@rows"/>
    <xsl:text>      </xsl:text>
    <g transform="translate({(position()-1)*$rowwidth},0)">
<!--      <xsl:text>&#10;        </xsl:text>
      <title><xsl:value-of select="@name"/></title> -->
      <xsl:text>&#10;</xsl:text>
      <xsl:apply-templates>
	<!-- Column set width is space for one row, with a small gap. -->
	<xsl:with-param name="width" select="$rowwidth - ($rowwidth * 0.05)"/>
	<xsl:with-param name="height" select="$bottom + $top"/>
      </xsl:apply-templates>
      <xsl:text>      </xsl:text>
    </g>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="col">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <!-- Calculate width of each column within the row width. -->
    <xsl:variable name="colwidth" select="$width div ../../@cols"/>
    <xsl:choose>
      <xsl:when test="/graph/@style = 'stacked'">
	<!-- Stacked means stack positive values above zero, negative
		values below zero.
	  -->
	<xsl:variable name="this" select="position()"/>
	<xsl:variable name="value" select="text()"/>
	<!-- This is tricky.  We shift the column vertically by the sum
		of all preceeding columns with the same sign, to stack
		positive values above zero and negative values below.
		To distinguish them, we multiply each previous value by
		the current one.  If they have the same sign, the product
		is positive, and we select based on that.
	  -->
	<xsl:text>        </xsl:text>
	<g fill="{@color}"
		transform="translate(0,{-sum(../col[position()&lt;$this
				and text() * $value > 0])})">
	  <xsl:text>&#10;          </xsl:text>
	  <title><xsl:value-of select="@name"/></title>
	  <xsl:text>&#10;          </xsl:text>
	  <path d="M 0,{$height} h {$width} v {number(text())*-1} h {$width*-1} Z"/>
	  <xsl:text>&#10;        </xsl:text>
	</g>
	  <xsl:text>&#10;</xsl:text>
      </xsl:when>
      <xsl:when test="/graph/@style = 'percent'">
	<!-- Percent means stack columns whose height is the fraction of
		the plot height that each column represents of the total
		of all columns.
	  -->
	<xsl:variable name="this" select="position()"/>
	<xsl:text>        </xsl:text>
	<g fill="{@color}"
		transform="translate(0,{-((sum(../col[position()&lt;$this])
					div sum(../col)) * $height)})">
	  <xsl:text>&#10;          </xsl:text>
	  <title><xsl:value-of select="@name"/></title>
	  <xsl:text>&#10;          </xsl:text>
	  <path d="M 0,{$height} h {$width} v {(text() div sum(../col))
			* $height * -1} h {$width * -1} Z"/>
	  <xsl:text>&#10;        </xsl:text>
	</g>
	  <xsl:text>&#10;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<!-- Normal means place each column side-by-side. -->
	<xsl:text>        </xsl:text>
	<g fill="{@color}" transform="translate({(position()-1)*$colwidth},0)">
	  <xsl:text>&#10;          </xsl:text>
	  <title><xsl:value-of select="@name"/></title>
	  <xsl:text>&#10;          </xsl:text>
	  <path d="M 0,{$height} h {$colwidth} v {number(text())*-1} h {$colwidth*-1} Z"/>
	  <xsl:text>&#10;        </xsl:text>
        </g>
	<xsl:text>&#10;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
