<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- XSLT stylesheet template fragment:
	Draws data lines for a line graph.
-->
<!-- $Id$ -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <xsl:template match="data">
    <!-- Shift data area right and up, above axes. -->
    <xsl:variable name="left" select="../x-axis/scale/@low"/>
    <xsl:variable name="top" select="../y-axis/scale/@low"/>
    <xsl:variable name="right" select="../x-axis/scale/@high"/>
    <xsl:variable name="bottom" select="../y-axis/scale/@high"/>
    <xsl:text>    </xsl:text>
    <svg x="15" y="0" width="85" height="85" stroke="black"
	stroke-width="{($right - $left) div 100}"
	viewBox="{$left} {$top} {$right - $left} {$bottom - $top}"
	preserveAspectRatio="none">
      <xsl:text>&#10;</xsl:text>
      <xsl:choose>
	<xsl:when test="../x-axis/scale/@type = 'enum'">
	  <xsl:call-template name="doline">
	    <xsl:with-param name="height" select="$bottom + $top"/>
	    <xsl:with-param name="xstep" select="($right - $left) div @rows"/>
	    <xsl:with-param name="xoffset" select="($right - $left) div @rows div 2"/>
	  </xsl:call-template>
	</xsl:when>
	<xsl:when test="../x-axis/scale/@type = 'numeric'">
	  <xsl:call-template name="doline">
	    <xsl:with-param name="height" select="$bottom + $top"/>
	    <xsl:with-param name="xstep" select="($right - $left) div @rows"/>
	  </xsl:call-template>
	</xsl:when>
      </xsl:choose>
      <xsl:text>    </xsl:text>
    </svg>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template name="doline">
    <xsl:param name="height"/>
    <xsl:param name="line" select="number(@cols)"/>
    <xsl:param name="xstep"/>
    <xsl:param name="xoffset" select="0"/>
    <xsl:if test="$line > 0">
      <!-- Only create a line if there are two or more numeric data points. -->
      <xsl:if test="count(row/col[position()=$line and not(@nonnumeric)])>1">
	<xsl:text>      </xsl:text>
	<g stroke="{row[1]/col[$line]/@color}" fill="none">
	  <xsl:text>&#10;        </xsl:text>
	  <title><xsl:value-of select="row[1]/col[$line]/@name"/></title>
	  <xsl:text>&#10;        </xsl:text>
	  <xsl:text disable-output-escaping="yes">&lt;path d="</xsl:text>
	  <xsl:apply-templates select="row">
	    <xsl:with-param name="height" select="$height"/>
	    <xsl:with-param name="line" select="$line"/>
	    <xsl:with-param name="xstep" select="$xstep"/>
	    <xsl:with-param name="xoffset" select="$xoffset"/>
	  </xsl:apply-templates>
	  <xsl:text disable-output-escaping="yes">"/&gt;</xsl:text>
	  <xsl:text>&#10;      </xsl:text>
	</g>
	<xsl:text>&#10;</xsl:text>
      </xsl:if>
      <xsl:call-template name="doline">
	<xsl:with-param name="height" select="$height"/>
	<xsl:with-param name="line" select="$line - 1"/>
	<xsl:with-param name="xstep" select="$xstep"/>
	<xsl:with-param name="xoffset" select="$xoffset"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template match="row">
    <xsl:param name="height"/>
    <xsl:param name="line"/>
    <xsl:param name="xstep"/>
    <xsl:param name="xoffset" select="0"/>
    <xsl:variable name="prev" select="position()-1"/>
    <!-- Only output this point if the value is numeric. -->
    <xsl:if test="not(col[$line]/@nonnumeric and /graph/@style = 'normal')">
      <xsl:choose>
	<!-- Start a new line at the first row or after non-numeric values. -->
	<xsl:when test="position() = 1 or 
			(../row[$prev]/col[$line]/@nonnumeric and
			 /graph/@style = 'normal')">
	  <xsl:text> M </xsl:text>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:text> L </xsl:text>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:value-of select="($xstep * position()) - $xoffset"/>
      <xsl:text>,</xsl:text>
      <!-- The Y value varies depending on the graph style. -->
      <xsl:choose>
	<xsl:when test="/graph/@style = 'stacked'">
	  <!-- Stacked means plot the sum of this and all previous columns. -->
	  <xsl:value-of select="$height -
		sum(col[position() &lt;= $line]/text())"/>
	</xsl:when>
	<xsl:when test="/graph/@style = 'percent'">
	  <!-- Percent means plot the fraction that this and all previous
		columns represents of the total of all columns.
	    -->
	  <xsl:value-of select="$height -
		sum(col[position() &lt;= $line]/text())
			div sum(col/text()) * $height"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="$height - col[$line]/text()"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
