<?xml version="1.0" encoding="ISO-8859-1"?>
 
<!-- XSLT stylesheet template to convert XML containing a result set
	into and HTML document containing a table.  This stylesheet
	accepts a number of parameters, described below.
 -->
<!-- $Id$ -->

<!-- Declare the stylesheet. -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0"
  >

  <!-- Declare parameters for this stylesheet.  They are as follows,
	with default values in square brackets.
	Title: a title to place above the table. []
	TextColor: the color to make all the text. [black]
	TextSize: the size to make text within the table. [-1]
	TextFace: the font face to use for all text. [sans-serif]
	BackgroundColor: the color for the page background. [white]
	RowColor1: the color for odd-numbered table rows. [#c0c0c0]
	RowColor2: the color for even-numbered table rows. [#6699cc]
	HeadingColor: the color for the heading of column names. [$RowColor2]
	RowNumbers: non-zero to include row numbers. [1]
	ColumnNames: non-zero to include column names. [1]
	RowLabel: a column name for the RowNumbers column. []
	NullValue: string used in place of fields marked as null. [&nbsp;]
    -->
  <xsl:param name="Title" select="''"/>
  <xsl:param name="TextColor" select="'black'"/>
  <xsl:param name="TextSize" select="'-1'"/>
  <xsl:param name="TextFace" select="'sans-serif'"/>
  <xsl:param name="BackgroundColor" select="'white'"/>
  <xsl:param name="RowColor1" select="'#c0c0c0'"/>
  <xsl:param name="RowColor2" select="'#6699cc'"/>
  <xsl:param name="HeadingColor" select="$RowColor2"/>
  <xsl:param name="RowNumbers" select="1"/>
  <xsl:param name="ColumnNames" select="1"/>
  <xsl:param name="RowLabel" select="'Row'"/>
  <xsl:param name="NullValue" select="'&amp;nbsp;'"/>

  <!-- Declare the output method: HTML. -->
  <xsl:output method="html" indent="yes"
              omit-xml-declaration="yes"
              version="1.0"
              encoding="UTF-8" />
  <xsl:strip-space elements="*"/>


  <!-- The default action is to ignore everything that we do not
	explicitly match.  This lets us ignore the column definitions
	and other stuff we do not need that may appear in the input data.
	Note that later templates do use the column definitions, but in
	a special mode used only to output the column names as a header row.
    -->
  <xsl:template match="node()"/>

  <!-- Main template: matches the RESULTS document element, outputs the
	HTML framework except for the contents of the table.
    -->
  <xsl:template match="/RESULTS">
    <html>
      <head><title><xsl:value-of select="$Title"/></title></head>
      <body bgColor="{$BackgroundColor}" text="{$TextColor}">
	<br/><br/>
	<center><font size="+1" face="{$TextFace}"
		color="{$TextColor}"><xsl:value-of
		select="$Title"/></font></center>
	<br/><br/>
	<table cellspacing="1" cellpadding="1" border="1" align="center"
		width="80%">
	  <!-- To output a row containing the column names, we select the
		column definition element in a special mode.  This mode
		causes templates to be invoked that would otherwise never
		match, and they output a row of the table getting the
		content in a different way than normal.
	    -->
	  <xsl:if test="boolean(number($ColumnNames))">
	    <xsl:apply-templates select="/RESULTS/COLDEF" mode="colnames"/>
	  </xsl:if>
	  <!-- Now do all the data rows.  Note that this will not invoke
		the templates we may have just used to create the
		column names.
	    -->
	  <xsl:apply-templates/>
	</table>
      </body>
    </html>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- Make a row of the table, alternating the colors. -->
  <xsl:template match="/RESULTS/ROW">
    <tr>
      <!-- Select the color for this row based on the row number, not
	   its position.  The position may vary depending on what comes
	   before it, but we know the row numbers will always start
	   with one.
	-->
      <xsl:choose>
	<xsl:when test="@num mod 2">
	  <xsl:attribute name="bgcolor">
	    <xsl:value-of select="$RowColor1"/>
	  </xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:attribute name="bgcolor">
	    <xsl:value-of select="$RowColor2"/>
	  </xsl:attribute>
	</xsl:otherwise>
      </xsl:choose>
      <!-- If we are numbering the rows, make the first cell of each row
	   contain the row number in bold.  Note that we pass a node-set
	   containing the <b> tags as the content, not just a text string.
	-->
      <xsl:if test="boolean(number($RowNumbers))">
	<xsl:call-template name="cell"> 
	  <xsl:with-param name="content">
	    <b><xsl:value-of select="@num"/></b>
	  </xsl:with-param>
	</xsl:call-template>
      </xsl:if>
      <!-- Do all the columns within this row. -->
      <xsl:apply-templates/>
    </tr>
  </xsl:template>

  <!-- For each data column, we invoke our cell template with the
	text content of the column.
    -->
  <xsl:template match="/RESULTS/ROW/*">
    <xsl:param name="content"/>
	<xsl:choose>
	  <xsl:when test="string(/RESULTS/COLDEF/COL[@name=current()/@name]/@type)='BLOB'">
		  <xsl:call-template name="cell">
			<xsl:with-param name="content" select="OMIT"/>
		  </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
		  <xsl:call-template name="cell">
			<xsl:with-param name="content" select="text()"/>
		  </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
  </xsl:template>

  <!-- The cell template is called by name to format the content of each
	cell within a row of the table.  It is passed the content to put
	into the cell, which may be a node-set or a string.  It wraps the
	content with the appropriate HTML markup to render the content
	based on our global parameters.  If the content is empty, we output
	a non-breaking space instead.  This causes Netscape to paint the
	background color for the cell, instead of ignoring it altogether
	and leaving a hole in the table.
    -->
  <xsl:template name="cell">
    <xsl:param name="content"/>
    <td>
      <font color="{$TextColor}" size="{$TextSize}" face="{$TextFace}">
	<xsl:choose>
	  <xsl:when test="boolean(@null)">
	    <!-- Use the NullValue parameter to represent fields explicitly
		marked as null.
	      -->
	    <xsl:value-of select="$NullValue"/>
	  </xsl:when>
	  <xsl:when test="string($content) = ''">
	    <!-- Getting the non-breaking space into the output as an entity
		was a pain.  I had to both escape the output AND represent
		the ampersand as an entity.  Apparently, entities are
		resolved within string literals.
	      -->
	    <xsl:value-of disable-output-escaping="yes" select="'&amp;nbsp;'"/>
	  </xsl:when>
	  <xsl:otherwise>
	  <!-- We use xsl:copy-of to include the content, because it is
		a node-set and may contain HTML markup we want to output.
	    -->
	    <xsl:copy-of select="$content"/>
	  </xsl:otherwise>
	</xsl:choose>
      </font>
    </td>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- This template is used to output the header row containing the names
	of the columns.  It uses a special mode so that it is NOT invoked
	by the default apply-templates in the main template.  This prevents
	the contents of the COLDEF element from appearing within the table.
	It makes a row using the HeadingColor, with the RowLabel if we
	are putting the row numbers in the first column.
    -->
  <xsl:template match="/RESULTS/COLDEF" mode="colnames">
    <tr bgcolor="{$HeadingColor}">
      <xsl:if test="boolean(number($RowNumbers))">
	<xsl:call-template name="cell"> 
	  <xsl:with-param name="content">
	    <b><xsl:value-of select="$RowLabel"/></b>
	  </xsl:with-param>
	</xsl:call-template>
      </xsl:if>
      <xsl:apply-templates mode="colnames"/>
    </tr>
  </xsl:template>

  <!-- This is used to get the column name from its attribute and put
	it into a cell.  It only gets invoked when we are creating the
	heading line of column names.
    -->
  <xsl:template match="/RESULTS/COLDEF/COL" mode="colnames">
    <xsl:param name="content"/>
    <xsl:call-template name="cell">
      <xsl:with-param name="content">
	<b><xsl:value-of select="@name"/></b>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>
