<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- XSLT stylesheet template to convert XML containing a result set
	into text of various formats.  There are parameters to control
	field padding, field and column delimiters, inclusion of a
	header containing the column names, a separating line of
	hyphens under the header, a leading column containing row
	numbers, and inter-column spacing.  By default, it outputs
	the data with the header of column names, numbers the rows,
	pads the fields to fixed width, and puts four spaces before
	each column.
 -->
<!-- $Id$ -->

<!-- Set up the stylesheet to use Java functions. -->

<!-- Declare the stylesheet. -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  exclude-result-prefixes="java"
  version="1.0"
  >

  <!-- Declare parameters for this stylesheet.  They are as follows,
	with default values in square brackets.
	RowNumbers: non-zero to include row numbers. [1]
	ColumnNames: non-zero to include column names. [1]
	RowLabel: a column name for the RowNumbers column. []
	RowLabelSize: the column width for row labels. [6]
	Margin: the number of spaces output before each column. [4]
	HeadingSeparator: non-zero to output row of hyphens after
		column names row, if ColumnNames is set. [1]
	FieldPadding: non-zero to pad fields to their display size. [1]
	BeginFieldDelim: string to place before each field value. []
	EndFieldDelim: string to place before each field value. []
	ColumnDelim: string to place between each column. []
	NullValue: string used in place of fields marked as null. [&nbsp;]
	ByteEncoding: character encoding to determine byte length of strings. []
    -->
  <xsl:param name="RowNumbers" select="1"/>
  <xsl:param name="ColumnNames" select="1"/>
  <xsl:param name="RowLabel" select="'Row'"/>
  <xsl:param name="RowLabelSize" select="6"/>
  <xsl:param name="Margin" select="4"/>
  <xsl:param name="HeadingSeparator" select="1"/>
  <xsl:param name="FieldPadding" select="1"/>
  <xsl:param name="BeginFieldDelim" select="''"/>
  <xsl:param name="EndFieldDelim" select="''"/>
  <xsl:param name="ColumnDelim" select="''"/>
  <xsl:param name="NullValue" select="''"/>
  <xsl:param name="ByteEncoding" select="''"/>

<!-- Declare the output method: TEXT. -->
  <xsl:output method="text" indent="no"
              omit-xml-declaration="yes"
              version="1.0"
              encoding="UTF-8" />
  <xsl:strip-space elements="*"/>

  <!-- Global variable of spaces for padding between fields.
	We made this really long so we can have really wide fields.
	This is done by concatenating a bunch of spaces until we
	have 2560 of them.
    -->
  <xsl:variable name="space40"
    select="'                                        '"/>
  <xsl:variable name="space160"
    select="concat(concat(concat($space40,$space40),$space40),$space40)"/>
  <xsl:variable name="space640"
    select="concat(concat(concat($space160,$space160),$space160),$space160)"/>
  <xsl:variable name="spaces"
    select="concat(concat(concat($space640,$space640),$space640),$space640)"/>


  <!-- The default action is to ignore everything that we do not
	explicitly match.  This lets us ignore the column definitions
	and other stuff we do not need that may appear in the input data.
    -->
  <xsl:template match="node()"/>

  <!-- Main template: matches the RESULTS document element.  This outputs
	the header and then all the data.
    -->
  <xsl:template match="/RESULTS">
    <!-- To output a row containing the column names, we select the
	column definition element in a special mode.  This mode
	causes templates to be invoked that would otherwise never
	match, and they output a row of the table getting the
	content in a different way than normal.
      -->
    <xsl:if test="boolean(number($ColumnNames))">
      <xsl:apply-templates select="/RESULTS/COLDEF" mode="colnames"/>
      <!-- Output a row of hyphens if asked for the heading separator. -->
      <xsl:if test="boolean(number($HeadingSeparator))">
	<xsl:apply-templates select="/RESULTS/COLDEF" mode="colnames">
	  <xsl:with-param name="separator" select="1"/>
	</xsl:apply-templates>
      </xsl:if>
    </xsl:if>
    <!-- Now do all the data rows.  Note that this will not invoke the
	templates we may have just used to create the column names.
      -->
    <xsl:apply-templates/>
  </xsl:template>

  <!-- Outputs each row of data.  If we are asked to, outputs the row
	number as the first cell in the row.  Ends each row with a newline.
    -->
  <xsl:template match="/RESULTS/ROW">
    <xsl:if test="boolean(number($RowNumbers))">
      <xsl:call-template name="cell">
	    <xsl:with-param name="content" select="@num"/>
	    <xsl:with-param name="width" select="$RowLabelSize"/>
	    <xsl:with-param name="nocoldelim" select="1"/>
      </xsl:call-template>
      <!-- Output the column delimiter to separate this column. -->
      <xsl:value-of disable-output-escaping="yes" select="$ColumnDelim"/>
    </xsl:if>
    <xsl:apply-templates/>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- For each column, we output its value using the cell template.
	We handle two forms.  If there is a NAME attribute, that contains
	the name of the column, otherwise the element name is the name
	of the column.  We use the name of the column to get its width
	from the column definitions, an the cell template to actually
	format the content of the cell.
    -->
  <xsl:template match="/RESULTS/ROW/*">
    <xsl:choose>
      <xsl:when test="boolean(@name)">
	    <xsl:variable name="name" select="@name"/>

        <xsl:choose>
	      <xsl:when test="string(/RESULTS/COLDEF/COL[@name=current()/@name]/@type)='BLOB'">

	        <xsl:call-template name="cell">
	          <xsl:with-param name="content" select="OMIT"/>
	          <xsl:with-param name="width" select="25"/>
	        </xsl:call-template>

          </xsl:when>
          <xsl:otherwise>

	        <xsl:call-template name="cell">
	          <xsl:with-param name="content" select="text()"/>
	          <xsl:with-param name="width"
		        select="/RESULTS/COLDEF/COL[@name=$name]/@displaysize"/>
	        </xsl:call-template>

          </xsl:otherwise>
        </xsl:choose>

      </xsl:when>

      <xsl:when test="name() = 'COL' and
			not(boolean(/RESULTS/COLDEF/COL[@name='COL']))">
        <xsl:choose>
	      <xsl:when test="string(/RESULTS/COLDEF/COL[@name=current()/@name]/@type)='BLOB'">
          </xsl:when>
          <xsl:otherwise>
          </xsl:otherwise>
        </xsl:choose>

	    <xsl:variable name="pos" select="position()"/>
	    <xsl:call-template name="cell">
	      <xsl:with-param name="content" select="text()"/>
	      <xsl:with-param name="width"
		    select="/RESULTS/COLDEF/COL[$pos]/@displaysize"/>
	    </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
	      <xsl:when test="string(/RESULTS/COLDEF/COL[@name=current()/@name]/@type)='BLOB'">
          </xsl:when>
          <xsl:otherwise>
          </xsl:otherwise>
        </xsl:choose>
	    <xsl:variable name="name" select="name()"/>
	    <xsl:call-template name="cell">
	      <xsl:with-param name="content" select="text()"/>
	      <xsl:with-param name="width"
		    select="/RESULTS/COLDEF/COL[@name=$name]/@displaysize"/>
	    </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- The cell template is called by name to format the content of each
	cell within a row of the table.  It is passed the content to put
	into the cell, which may be a node-set or a string; the width of
	the output field in characters; and a flag indicating if the data
	is to be replaced by hyphens; and a flag indicating if the column
	delimiter should be suppressed.  It truncates or pads the content
	with spaces to make it that size, and prepends the Margin.
	The hyphen feature is used for the heading separator.
    -->
  <xsl:template name="cell">
    <xsl:param name="content"/>
    <xsl:param name="width"/>
    <xsl:param name="separator"/>
    <xsl:param name="nocoldelim"/>

    <!-- Output the margin before each cell. -->
    <xsl:value-of select="substring($spaces, 1, $Margin)"/>

    <!-- Output the column delimiter if we are not on the first column. -->
    <xsl:if test="not(boolean($nocoldelim)) and position() > 1">
      <xsl:value-of disable-output-escaping="yes" select="$ColumnDelim"/>
    </xsl:if>

    <!-- Output the beginning field delimiter. -->
    <xsl:value-of disable-output-escaping="yes" select="$BeginFieldDelim"/>

    <!-- Decide how to render the field content.  We could output the
	content as it is, replace values marked as null with the null
	value, or replace it with hyphens for the heading separator.
      -->
    <xsl:choose>
      <xsl:when test="boolean($separator)">
	    <xsl:call-template name="value">
	      <!-- Heading separator: a string of hyphens to fill the width. -->
	      <xsl:with-param name="content"
		    select="substring(translate($spaces,' ','-'), 1, $width)"/>
	      <xsl:with-param name="width" select="$width"/>
	    </xsl:call-template>
      </xsl:when>
      <xsl:when test="boolean(@null)">
	    <xsl:call-template name="value">
	      <xsl:with-param name="content" select="$NullValue"/>
	      <xsl:with-param name="width" select="$width"/>
	    </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
	    <xsl:call-template name="value">
	      <xsl:with-param name="content" select="$content"/>
	      <xsl:with-param name="width" select="$width"/>
	    </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>

    <!-- Output the ending field delimiter. -->
    <xsl:value-of disable-output-escaping="yes" select="$EndFieldDelim"/>
  </xsl:template>

  <!-- Template to output the value of a single cell.  This is where we
	handle the truncation or padding of the cell value.
    -->
  <xsl:template name="value">
    <xsl:param name="content"/>
    <xsl:param name="width"/>
    <xsl:choose>
      <xsl:when test="boolean(number($FieldPadding))">
	    <xsl:variable name="numBytes"
		    select="java:com.rocketsoftware.xml.xslt.XslUtil.getByteLen($content, $ByteEncoding)"/>
        <xsl:choose>
	      <xsl:when test="string(/RESULTS/COLDEF/COL[@name=current()/@name]/@type)='BLOB'">

			<xsl:variable name="charstopad" select="25 - $numBytes"/>

	        <xsl:value-of disable-output-escaping="yes"
		        select="substring( string( $content ), 1, 25 )"/>
	        <xsl:value-of
	            select="substring( $spaces, 1, $charstopad )"/>

          </xsl:when>
          <xsl:otherwise>

			<xsl:variable name="charstopad" select="$width - $numBytes"/>

	        <!-- Fixed width: truncate then pad content to field width. -->
	        <xsl:value-of disable-output-escaping="yes"
		        select="substring( string($content), 1, $width )"/>
	        <xsl:value-of
	            select="substring( $spaces, 1, $charstopad )"/>

          </xsl:otherwise>
        </xsl:choose>

      </xsl:when>
      <xsl:otherwise>
	    <!-- Variable width: output entire content as it is. -->
	    <xsl:value-of disable-output-escaping="yes" select="string($content)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- This template is used to output the header row containing the names
	of the columns.  It uses a special mode so that it is NOT invoked
	by the default apply-templates in the main template.  This prevents
	the contents of the COLDEF element from appearing within the output.
	It begins the row with the RowLabel if we are putting the row numbers
	in the first column.
    -->
  <xsl:template match="/RESULTS/COLDEF" mode="colnames">
    <xsl:param name="separator"/>
    <xsl:if test="boolean(number($RowNumbers))">
      <xsl:call-template name="cell">
	<xsl:with-param name="content" select="$RowLabel"/>
	<xsl:with-param name="width" select="$RowLabelSize"/>
	<xsl:with-param name="separator" select="$separator"/>
	<xsl:with-param name="nocoldelim" select="1"/>
      </xsl:call-template>
      <!-- Output the column delimiter to separate this column. -->
      <xsl:value-of disable-output-escaping="yes" select="$ColumnDelim"/>
    </xsl:if>
    <xsl:apply-templates mode="colnames">
      <xsl:with-param name="separator" select="$separator"/>
    </xsl:apply-templates>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- This is used to get the column name from its attribute and put
	it into a cell.  It only gets invoked when we are creating the
	heading line of column names.
    -->
  <xsl:template match="/RESULTS/COLDEF/COL" mode="colnames">
    <xsl:param name="separator"/>
    <xsl:call-template name="cell">
      <xsl:with-param name="content" select="@name"/>
      <xsl:with-param name="width" select="@displaysize"/>
      <xsl:with-param name="separator" select="$separator"/>
    </xsl:call-template>
  </xsl:template>

</xsl:stylesheet>
