// =======================================
// Web Query Catalog DDL for z/OS
// =======================================

CREATE DATABASE WEBQUERY;

CREATE TABLESPACE WQSPACE IN WEBQUERY
	USING STOGROUP SYSDEFLT
	PRIQTY 16000
	SECQTY 8000;

CREATE TABLE WQ.INDEX_TABLE
(
	CREATEDTS TIMESTAMP NOT NULL,
	CREATEDIX SMALLINT NOT NULL,
	CREATOR VARCHAR(32) NOT NULL,
	NAME VARCHAR(128) NOT NULL,
	DESCRIPTION VARCHAR(255),
	CATEGORY VARCHAR(32) NOT NULL,
	VERSION SMALLINT NOT NULL,
	TYPE SMALLINT NOT NULL,
	PUB SMALLINT NOT NULL WITH DEFAULT 0,
	CREATEDBY VARCHAR(32) NOT NULL,
	PRIMARY KEY (CREATEDTS, CREATEDIX)
) IN WEBQUERY.WQSPACE CCSID EBCDIC;

CREATE UNIQUE INDEX WQ.INDEX_I1
    ON WQ.INDEX_TABLE
(
     CREATEDTS, CREATEDIX
);

CREATE UNIQUE INDEX WQ.INDEX_I2
    ON WQ.INDEX_TABLE
(
 	CREATOR,
	NAME,
	TYPE
);

COMMENT ON TABLE WQ.INDEX_TABLE
    IS 'DB2 Web Query Tool Index Table';


CREATE TABLE WQ.STORE_TABLE
(
	CREATEDTS TIMESTAMP NOT NULL,
	CREATEDIX SMALLINT NOT NULL,
	SEQUENCE INTEGER NOT NULL,
	OBJECT VARCHAR(3500) FOR BIT DATA NOT NULL,

	FOREIGN KEY (CREATEDTS, CREATEDIX)
		REFERENCES WQ.INDEX_TABLE ON DELETE CASCADE
) IN WEBQUERY.WQSPACE CCSID EBCDIC;

COMMENT ON TABLE WQ.STORE_TABLE
    IS 'DB2 Web Query Tool Storage Table';

CREATE UNIQUE INDEX WQ.INDEX_I3
    ON WQ.STORE_TABLE
(
 	CREATEDTS,
	CREATEDIX,
	SEQUENCE
);


CREATE TABLE WQ.SMTP
(
	EMAIL VARCHAR(80) NOT NULL,
	USRNAME VARCHAR(20) NOT NULL,
	PASSWD VARCHAR(50),
	AUTHDATE TIMESTAMP,

	PRIMARY KEY (EMAIL)
) IN WEBQUERY.WQSPACE CCSID EBCDIC;

COMMENT ON TABLE WQ.SMTP
    IS 'DB2 Web Query Tool SMTP Process Table';

CREATE UNIQUE INDEX WQ.SMTP_I1
    ON WQ.SMTP
(
	EMAIL
);
